/*
  File name: CSDLClass.h
  Copyright: (C) 2005 - 2007 Tomasz Kazmierczak

  Creation date: 01.07.2005 12:52
  Last modification date: 28.08.2007

  This file is part of Cytadela

 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *

*/

#ifndef _CSDLCLASS_H_
#define _CSDLCLASS_H_

#include <SDL/SDL.h>
#include <SDL/SDL_mixer.h>
#include "farread.h"

#define CSDL_DEFAULT_SCREEN_W      640
#define CSDL_DEFAULT_SCREEN_H      480
#define CSDL_DEFAULT_SCREEN_W_TXT  "640"
#define CSDL_DEFAULT_SCREEN_H_TXT  "480"

#define CSDL_LOAD_SOUND_FAILED     -1
#define CSDL_AUDIO_NOT_INITIALIZED -2

enum VideoMode {
	MODE_2D,   //2D hw surface
	MODE_3D    //OpenGL
};

class ScreenSize {
public:
	char    *szResolution[2];
	int32_t  iResolution[2];
	bool     bAvailable;
};

class CSDLClass;

class CImagesArray {
friend class CSDLClass;
private:
	SDL_Surface **mImageSurface;
	ImgDims      *mImageDims;
	uint32_t      mImagesCount;
public:

	CImagesArray() :  mImageSurface(0), mImageDims(0), mImagesCount(0) {};
	~CImagesArray() { freeIA(); };

	void freeIA();
	bool loadImg(char* szFarFileName, char* szImgFileName, uint32_t *pImgNum);

	const ImgDims *getImageDimensions(uint32_t uiImageNum);
};

typedef class {
public:
	SDLKey eKey[5];     //eKey and bStatus are used to handle key up/down events
	bool   bStatus[5];  // which just occured (we could use pKeyMap for this,
						// but then we would have to check the previous key map
						// status - it is much simplier to just check for events
						// which have just occured) - good to handle options
						// switching
	uint8_t *pKeyMap;   //pressed/unpressed keys state - a good way to handle
						// player's
						//movement and weapons/tools usage
	int16_t siMouseX;
	int16_t siMouseY;
	bool    bLMB;
	bool    bRMB;
	bool    bJBtn;
	bool    bJoyUp;
	bool    bJoyDown;
	bool    bJoyLeft;
	bool    bJoyRight;
} InputArray;

//don't need more sounds, and fixed size array is easier to handle than dynamical one
#define MAX_SOUNDS_COUNT     25

class CSDLClass {
private:
	SDL_Surface  *mScreen;
	SDL_Joystick *mJoystick;
	Mix_Music    *mMusic;
	Mix_Chunk    *mSounds[MAX_SOUNDS_COUNT];
	bool          mAudio;  //if audio is available
	int32_t       mLoadedSoundsCount;
	int16_t       mMenuMouseXPos;
	int16_t       mMenuMouseYPos;
	uint32_t      mLastMouseInputTime;
	uint32_t      mLastJoyInputTime;

	int32_t       mScreenSizesCount;
	ScreenSize   *mScreenSizes;

	CSDLClass();
	~CSDLClass() { free(); };
public:
	CImagesArray mImgArray;

	//init/free
	static CSDLClass *instance();

	bool init();
	void free();
	bool switchVideoMode(int32_t iWidth, int32_t iHeight, VideoMode eMode);

	//input
	void delayInput();
	void clearEventQueue();
	SDLKey readInputMenu();
	void readInputGame(InputArray *pInputArray);

	//time
	uint32_t getTicks() { return SDL_GetTicks(); };

	void delay(uint32_t dwMs) { SDL_Delay(dwMs); };

	//display
	void displayImage(uint32_t dwImgNum, SDL_Rect *pSrcRect, int16_t iX, int16_t iY);
	void displayText(uint32_t dwImgNum, const char *szText, int16_t iX, int16_t iY);
	void blackScreen();
	void drawFrame() { SDL_Flip(mScreen); };
	void swapBuffers() { SDL_GL_SwapBuffers(); };
	int32_t getScreenSizesCount() { return mScreenSizesCount; };
	const ScreenSize *getScreenSizes() { return mScreenSizes; };
	int32_t findFirstAvailableScreenSize(int32_t iStart);

	//audio
	bool loadMusic(char *szFileName);
	void freeMusic();
	void playMusic(int32_t iLoops);
	void stopMusic();
	int32_t loadSound(char *szFileName);
	void freeSounds();
	void playSoundFadeIn(int32_t iSoundNumber, int32_t iLoops, int32_t iFadeInTime);
	void playSound(int32_t iSoundNumber, int32_t iChannel, int32_t volume);
	void playSound(int32_t iSoundNumber, int32_t iChannel);
	void stopSound(int32_t iSoundNumber, int32_t iFadeOutTime);
};

#endif
