/*
  File name: GOInterface.cpp
  Copyright: (C) 2007 Tomasz Kazmierczak

  Creation date: 23.04.2007
  Last modification date: 8.09.2007

  This file is part of Cytadela

 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *

*/

#include "GOInterface.h"
#include <cassert>

GOInterface *createGOInterface()
{
	return new GOInterface;
}

GOInterface::~GOInterface()
{
	while(not mGameObjects.empty())
	{
		if(mGameObjects.begin()->second)
			delete mGameObjects.begin()->second;
		mGameObjects.erase(mGameObjects.begin());
	}
	if(mMapData)
		delete[] mMapData;
}

GameObject *GOInterface::createGameObject(GameObjectType type, Mesh3Dg *mesh)
{
	GameObject *object;
	switch(type)
	{
		case GOTYPE_WALL:
			object = new GOWall(mesh);
			break;
		case GOTYPE_DOOR:
			object = new GODoor(mesh);
			break;
		case GOTYPE_SPRITE:
			object = new GOSprite(mesh);
			break;
		default:
			 return 0;
	}
	if(not addGameObject(object))
	{
		delete object;
		return 0;
	}
	return object;
}

TemporaryObject *GOInterface::createTempGameObject(Mesh3Dg *mesh, uint32_t time)
{
	TemporaryObject *object = new TemporaryObject(mesh, time);
	if(not addGameObject((GameObject *)object))
	{
		delete object;
		return 0;
	}
	return object;
}

TemporaryObject *GOInterface::createTempGameObject(Mesh3Dg *mesh, float range, Vector3Dg direction)
{
	TemporaryObject *object = new TemporaryObject(mesh, range, direction);
	if(not addGameObject((GameObject *)object))
	{
		delete object;
		return 0;
	}
	return object;
}

bool GOInterface::destroyGameObject(GameObject *object)
{
	if(not remGameObject(object))
		return false;

	delete object;
	return true;
}

bool GOInterface::addGameObject(GameObject *object)
{
	//first check if there is already an object with this uid
	GameObjectsMap::iterator i = mGameObjects.find(object->getMesh()->getUID());
	//if there is
	if(i != mGameObjects.end())
//TODO: some bug handling? is it necesary here? (the function can fail only here)
		return false;

	mGameObjects[object->getMesh()->getUID()] = object;
	return true;
}

bool GOInterface::remGameObject(GameObject *object)
{
	//first check if there is such object
	GameObjectsMap::iterator i = mGameObjects.find(object->getMesh()->getUID());
	//if there isn't
	if(i == mGameObjects.end())
		return false;

	mGameObjects.erase(i);
	return true;
}

uint16_t *GOInterface::allocMapData(uint32_t size)
{
	if(mMapData)
		delete[] mMapData;

	mMapDataSize = size;
	mMapData = new uint16_t[size];
	return mMapData;
}

GameObject *GOInterface::getGameObject(Mesh3DgUID muid)
{
#ifdef CHECK_ASSERTS
	//first check if there is an object with this uid
	GameObjectsMap::iterator i = mGameObjects.find(muid);
	//if there isn't
	assert(i != mGameObjects.end());
#endif

	return mGameObjects[muid];
}

bool TemporaryObject::update(uint32_t time)
{
	mTimeCounter += time;
	if(mTimeCounter <= mAnimStartDelay)
		return true;
	else
	{
		getMesh()->setProperty(MRP_RENDERABLE, true);
		mTimeCounter -= mAnimStartDelay;
		mAnimStartDelay = 0;
	}

	Mesh3Dg *mesh = getMesh();
	MeshAnim3Dg *animation = &mesh->mAnimation;

	if(animation->dwTexAnimTime)
	{
		//set next material
		mesh->mMaterialNumber = animation->dwBeginMtl +
			(mTimeCounter * animation->dwNumMtls) / animation->dwTexAnimTime;
	}

	if(mTimeCounter < mLifeTime)
		return true;
	else return false;
}

bool TemporaryObject::update(uint32_t time, float distaince)
{
	mDistaince += distaince;
	mTimeCounter += time;
	if(mTimeCounter <= mAnimStartDelay)
		return true;
	else
	{
		getMesh()->setProperty(MRP_RENDERABLE, true);
		mTimeCounter -= mAnimStartDelay;
		mAnimStartDelay = 0;
	}

	Mesh3Dg *mesh = getMesh();
	MeshAnim3Dg *animation = &mesh->mAnimation;

	if(animation->dwTexAnimTime)
	{
		//if texture display time went out of range then substract
		//as long as it gets back into the range
		while(mTimeCounter >= animation->dwTexAnimTime)
			mTimeCounter -= animation->dwTexAnimTime;
		//set next material
		mesh->mMaterialNumber = animation->dwBeginMtl +
			(mTimeCounter * animation->dwNumMtls) / animation->dwTexAnimTime;
	}

	if(mDistaince < mRange)
		return true;
	else return false;
}
